;***********************************************************
;*                    I2C_IO.ASM  V1.0                     *
;* LED-Lauflicht gesteuert ueber 8-Bit-I/O-Port an I2C-Bus *
;* Mikrocontroller: 8051-kompatibel, f=11,0592 MHz         *
;* I2C-I/O-Baustein: PCF8574                               *
;* P0-P6: 7 LEDs, leuchten jeweils bei LOW-Pegel           *
;* P7: Jumper gegen GND, Jumper geschlossen = Mode 0       *
;*                       Jumper offen = Mode 1             *
;***********************************************************
;* (c) 2001, Michael Gaus                                  *
;***********************************************************
;
; P3.  7       5
;     SDA     SCL


sda equ p3.7
scl equ p3.5

i2c_ack_in equ 0       ; vom Slave gesendetes Acknowledge-Bit
i2c_ack_out equ 1      ; an den Slave zu sendendes Acknowledge-Bit
                       ; jeweils 0=Ack, 1=kein Ack

MODE equ 0             ; Bit fuer Lauflichtmodus (Bitbereich)
i2c_adr equ 40h        ; I2C-Adresse des PCF8574

STEPS_LED0 equ 12      ; Anzahl der Schritte fuer Mode 0
STEPS_LED1 equ 14      ; Anzahl der Schritte fuer Mode 1


reset:
       call read_mode      ; Lauflichtmodus auswerten
       jb MODE,mode_1

mode_0:
       mov dptr,#led0
       mov r7,#STEPS_LED0
m0_loop:
       call i2c_start      ; I2C-Start
       mov a,#i2c_adr      ; Adresse PCF8574
       call i2c_send
       clr a
       movc a,@a+dptr      ; lese Daten aus Tabelle
       xrl a,#0FFh         ; Bits invertieren (LEDs leuchten bei LOW-Pegel)
       call i2c_send
       call i2c_stop       ; I2C-Stop
       call delay          ; Verzoegerung
       jb MODE,mode_1
       inc dptr
       djnz r7,m0_loop
       jmp mode_0


mode_1:
       mov dptr,#led1
       mov r7,#STEPS_LED1
m1_loop:
       call i2c_start      ; I2C-Start
       mov a,#i2c_adr      ; Adresse PCF8574
       call i2c_send
       clr a
       movc a,@a+dptr      ; lese Daten aus Tabelle
       xrl a,#0FFh         ; Bits invertieren (LEDs leuchten bei LOW-Pegel)
       call i2c_send
       call i2c_stop       ; I2C-Stop
       call delay          ; Verzoegerung
       jnb MODE,mode_0
       inc dptr
       djnz r7,m1_loop
       jmp mode_1



i2c_start:                 ; I2C Startbedingung
       setb sda
       nop
       setb scl
       call i2c_wait

       clr sda             ; Data auf 0 (Start)
       nop
       clr scl             ; Clock auf 0
       ret


i2c_stop:                  ; I2C Stopbedingung
       setb scl            ; Clock auf 1
       nop
       setb sda            ; Data auf 1 (Stop)
       ret



i2c_send:                  ; 1 Datenbyte (Akku) ueber I2C senden
       push 3              ; R3 auf Stack sichern
       push acc            ; Akku auf Stack sichern
       mov r3,#8
i2c_sloop:
       rlc a
       mov sda,c
       nop
       setb scl
       call i2c_wait
       clr scl
       djnz r3,i2c_sloop

       nop
       setb sda
       nop
       setb scl
       call i2c_wait           
       mov c,sda            ; Acknowledge-Bit lesen
       mov i2c_ack_in,c
       clr scl
       nop
       clr sda
       pop acc
       pop 3
       ret



i2c_read:                  ; 1 Datenbyte ueber I2C empfangen (Akku)
       push 3              ; R3 auf Stack sichern
       mov r3,#8
       clr a
       setb sda
i2c_rloop:
       call i2c_wait
       setb scl
       nop
       mov c,sda
       rlc a
       nop
       clr scl
       djnz r3,i2c_rloop

       mov c,i2c_ack_out   ; Acknowledge-Bit senden
       mov sda,c
       nop
       setb scl
       call i2c_wait
       clr scl
       nop
       clr sda
       pop 3
       ret


i2c_wait:               ; Verzoegerung fuer I2C (CALL + RET -> 4us bei 12MHz)
       ret



read_mode:
       push acc
       call i2c_start      ; I2C-Start
       mov a,#(i2c_adr+1)  ; Adresse PCF8574 (Lesezugriff)
       call i2c_send
       setb i2c_ack_out    ; kein Acknowledge senden
       call i2c_read
       rlc a
       mov MODE,c
       pop acc
       ret



delay:
       push 6
       push 7
       call read_mode      ; Lauflichtmodus auswerten
       mov r7,#07Fh
dloop1:
       mov r6,#0FFh
dloop2:
       nop
       nop
       djnz r6,dloop2
       djnz r7,dloop1
       pop 7
       pop 6
       ret

led0:                      ; Tabelle fuer Mode 0
       db 01h,02h,04h,08h,10h,20h,40h,20h,10h,08h,04h,02h

led1:                      ; Tabelle fuer Mode 0
       db 00h,01h,03h,07h,0Fh,1Fh,3Fh,7Fh,7Eh,7Ch,78h,70h,60h,40h

       END
